/* FM for VEF
 * Reads Heterodin frequency on LW and maps it to FM
 * Evgeniy Gichev 2021
 
- QN8006 module:

Top view:
+-10--9--8--7--6-+
|  +------+  ++  |
|  | QN   |  ||  |
|  | 8006 |  ||  |
|  +------+  ++  |
+--1--2--3--4--5-+

1 ----> Arduino SDA
2 ----> Arduino SCL
3 ----> Audio in (left channel)
4 ----> Audio in (right channel)
5 ----> +5V
6 ----> GND
7 ----> Audio out (right channel)
8 ----> Audio out (left channel)
10 ---> Antenna

 */
#include <FreqCount.h>
#include <Wire.h>
#include <QN8006Radio.h>
QN8006Radio radio = QN8006Radio();
double old_freq = 0;
boolean muted = false;
byte search_trshold = 36;
byte stereo = 1;
byte mono = 0;
int button = 13;
boolean old_status = false;
boolean auto_search = false;

void setup() {
  Serial.begin(57600);
  FreqCount.begin(10);
  Wire.begin();
  Wire.setClock(400000);
  radio.initReceive(search_trshold, mono);
  pinMode(button, INPUT);
  boolean old_status = false;
}

void loop() {
//  pinMode(6,OUTPUT);
unsigned char stat[6];
unsigned int signal_level;
  
  if (FreqCount.available()) {
    unsigned long count = FreqCount.read();
    int frequencyInt = map(count,6100 , 8950, 8750, 10800);
    double frequency = frequencyInt/100.0f;
    if  (digitalRead(button)) old_status = true;
      else {
        if (old_status) {
//          Serial.println("Search");
          radio.searchUp(stat);
          old_status = false;
          auto_search = true;
        }
      }
      
    if (frequency - old_freq >0.04f || old_freq - frequency >0.04f) {
      if (auto_search) {
        radio.initReceive(search_trshold, mono);
        auto_search = false;
      }

     
//     Serial.println(frequency);
      radio.setFrequency(frequency);
      old_freq = frequency;
      
 
/*  
      
      if (radio.getStat(stat)) {
        double current_freq = (stat[0] +  stat[1] * 256) * 0.05 + 76;
        signal_level = stat [2];       
        Serial.print("Heterodin freq = ");
        Serial.print(count);
        Serial.print(" FM freq = ");
        Serial.print(current_freq);
        Serial.print(" RSSI= ");
        Serial.println(signal_level);

      if (signal_level < 24 && !muted) {
          radio.mute();
          muted = true;
         }
         if (signal_level >= 24 && muted){
          radio.unmute();
          muted = false;
         }

        }
         */
              
     }



  }
}
